import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:flutter/material.dart';
import '../../abg_utils.dart';

List<CategoryData> categories = [];

List<String> getCategories(List<String> val, String locale, List<CategoryData> category){
  List<String> ret = [];
  for (var item in val) {
    for (var item2 in category)
      if (item == item2.id) {
        ret.add(getTextByLocale(item2.name, locale));
        break;
      }
  }
  return ret;
}

String getCategoryNames(List<String> ids){
  var _text = "";
  for (var item in ids) {
    var t = getCategoryNameById(item);
    if (t.isNotEmpty){
      if (_text.isNotEmpty)
        _text = "$_text, ";
      _text = "$_text$t";
    }
  }
  return _text;
}

List<String> getServiceCategories(List<String> val){
  List<String> ret = [];
  for (var item in val) {
    for (var item2 in categories)
      if (item == item2.id) {
        ret.add(getTextByLocale(item2.name, locale));
        break;
      }
  }
  return ret;
}

String getCategoryNameById(String id){
  for (var item in categories)
    if (item.id == id)
      return getTextByLocale(item.name, locale);
  return "";
}

Future<String?> loadCategory() async{
  try{
    var querySnapshot = await FirebaseFirestore.instance.collection("category").get();
    categories = [];
    for (var result in querySnapshot.docs) {
      var _data = result.data();
      var t = CategoryData.fromJson(result.id, _data);
      categories.add(t);
    }
    // addStat("categories", parent.categories.length);
  }catch(ex){
    return "loadCategory " + ex.toString();
  }
  return null;
}

bool ifCategoryHaveSubcategories(String id){
  for (var item in categories)
    if (item.parent == id)
      return true;
  return false;
}

class CategoryData {
  String id;
  List<StringData> name;
  List<StringData> desc;
  bool visible;
  bool visibleCategoryDetails;

  String localFile = "";
  String serverPath = "";

  Color color;
  String parent;
  bool select = false;
  String assetFile = "";
  final dataKey = GlobalKey();
  final dataKey2 = GlobalKey();

  CategoryData({required this.id, required this.name,
    required this.localFile, required this.serverPath,
    required this.color, required this.parent,
    required this.visible, required this.visibleCategoryDetails,
    required this.desc, this.assetFile = ""});

  factory CategoryData.createEmpty(){
    return CategoryData(id: "", name: [], localFile: "", serverPath: "",
        color: Colors.green, parent: "", visible: true, visibleCategoryDetails: true,
        desc: []);
  }

  Map<String, dynamic> toJson() => {
    'name' : name.map((i) => i.toJson()).toList(),
    'desc' : desc.map((i) => i.toJson()).toList(),
    'visible' : visible,
    'visibleCategoryDetails' : visibleCategoryDetails,
    'localFile': localFile,
    'serverPath' : serverPath,
    'color': color.value.toString(),
    'parent' : parent,
  };

  factory CategoryData.fromJson(String id, Map<String, dynamic> data){
    List<StringData> _name = [];
    if (data['name'] != null)
      for (var element in List.from(data['name'])) {
        _name.add(StringData.fromJson(element));
      }
    List<StringData> _desc = [];
    if (data['desc'] != null)
      for (var element in List.from(data['desc'])) {
        _desc.add(StringData.fromJson(element));
      }
    return CategoryData(
      id: id,
      name: _name,
      localFile: (data["localFile"] != null) ? data["localFile"] : "",
      serverPath: (data["serverPath"] != null) ? data["serverPath"] : "",
      color: (data["color"] != null) ? toColor(data["color"]) : Colors.red,
      parent: (data["parent"] != null) ? data["parent"] : "",
      visible: (data["visible"] != null) ? data["visible"] : true,
      visibleCategoryDetails: (data["visibleCategoryDetails"] != null) ? data["visibleCategoryDetails"] : true,
      desc: _desc,
    );
  }
}


